`timescale 1ns/1ps

`include "vme64x_bfm.svh"
`include "svec_vme_buffers.svh"

`include "vhd_wishbone_master.svh"
`include "logger.svh"
`include "mock_turtle_driver.svh"
`include "masterfip_driver.svh"

`define VME_OFFSET      'h8000_0000
`define MT_BASE         `VME_OFFSET + 'h0004_0000



module main;

    parameter g_fast_load_cpu_firmware = 0;
   
    reg rst_n = 0;
    reg clk_sys = 0;
    reg clk_test = 0;
    reg spi_clk;
    reg rst;

    logic spi_cs_n = 1'b1;
    logic spi_mosi = 1'b0;
    logic spi_miso = 1'b0;
    logic [31:0] mosi_data;
    logic [31:0] miso_data;
    logic [31:0] test_miso_data;

    logic [4:0]  negedge_cnt    = 'h0;
    logic [4:0]  posedge_cnt    = 'h0;

    logic tmp_spi_clk = 1'b0;

    logic [31:0] q_mosi_data[$];
    logic [31:0] q_miso_data[$];

    always #8ns clk_sys <= ~clk_sys;
    always #100ns tmp_spi_clk <= ~tmp_spi_clk; // 500ns for 1MHz, 100ns for 5MHz, 25ns for 20MHz

    assign spi_clk = ~spi_cs_n ? tmp_spi_clk : 0;

    initial begin
        repeat(20) @(posedge clk_sys);
        rst_n = 1;
    end
 
    IVME64X VME(rst_n);

    `DECLARE_VME_BUFFERS(VME.slave);

    bit [4:0] slot_id = 8;


    // the Device Under Test   
    svec_masterfip_mt_urv
        #(
          .g_simulation(1)
        )
    DUT (
      // Clock, Reset
      .rst_n_i             (rst_n),
      .clk_125m_pllref_p_i (clk_sys),
      .clk_125m_pllref_n_i (~clk_sys),
      // ERTEC SPI Interface
      .ertec_spi_clk_i     (spi_clk),
      .ertec_spi_mosi_i    (spi_mosi),
      .ertec_spi_cs_n_i    (spi_cs_n),
      .ertec_spi_miso_o    (spi_miso),
      // VME Interface
      .vme_sysreset_n_i    (VME_RST_n),
      .vme_as_n_i          (VME_AS_n),
      .vme_write_n_i       (VME_WRITE_n),
      .vme_am_i            (VME_AM),
      .vme_ds_n_i          (VME_DS_n),
      .vme_gap_i           (^slot_id),
      .vme_ga_i            (~slot_id),
      .vme_berr_o          (VME_BERR),
      .vme_dtack_n_o       (VME_DTACK_n),
      .vme_retry_n_o       (VME_RETRY_n),
      .vme_retry_oe_o      (VME_RETRY_OE),
      .vme_lword_n_b       (VME_LWORD_n),
      .vme_addr_b          (VME_ADDR),
      .vme_data_b          (VME_DATA),
      .vme_irq_o           (VME_IRQ_n),
      .vme_iack_n_i        (VME_IACK_n),
      .vme_iackin_n_i      (VME_IACKIN_n),
      .vme_iackout_n_o     (VME_IACKOUT_n),
      .vme_dtack_oe_o      (VME_DTACK_OE),
      .vme_data_dir_o      (VME_DATA_DIR),
      .vme_data_oe_n_o     (VME_DATA_OE_N),
      .vme_addr_dir_o      (VME_ADDR_DIR),
      .vme_addr_oe_n_o     (VME_ADDR_OE_N)
    );

    IMockTurtleIRQ IrqMonitor (`MT_ATTACH_IRQ(DUT.cmp_mock_turtle_urv));
      IVHDWishboneMaster U_WBMaster
      (
          .clk_i(DUT.clk_62m5_sys),
          .rst_n_i(DUT.rst_n_sys)
      );

    task automatic checkSDBSignature( CBusAccessor acc );
        automatic uint64_t sig;
        automatic Logger l = Logger::get();

        const uint32_t SDB_ID = 'h5344422d;
        acc.read(0, sig);
        l.startTest("Check Wishbone communication");
        l.msg(0, $sformatf("SDB Signature: 0x%x", sig));
        if(sig != SDB_ID)
            l.fail($sformatf("Incorrect SDB signature (got 0x%x, expected 0x%x)", sig, SDB_ID));
        else
            l.pass();   
    endtask // check_sdb_signature

    task automatic checkCPUCommunication(MasterfipDriver drv);
        automatic Logger l = Logger::get();
        automatic MQueueMsg request = new (0, 5, '{}, '{});
        automatic int status;

        automatic mstrfip_hw_speed_trtlmsg response = new ();
        automatic MQueueMsg tmp = response;
        response.core = 0;
        response.slot = 5;


        request.header.flags  = `TRTL_HMQ_HEADER_FLAG_RPC | `TRTL_HMQ_HEADER_FLAG_SYNC;
        request.header.msg_id = MSTRFIP_CMD_GET_BITRATE;
        request.header.len = 2;
        request.data[0] = MSTRFIP_CMD_GET_BITRATE;

        $display("request [1]",request.tostring());
        $display("response [1]",response.tostring());

        drv.sendAndReceiveSync(request, tmp, 100, status );

        $display("tmp ",tmp.tostring());

        // TODO: The right one, for bitrate is the response.header.msg_id, because
        // this information is in the header part of the message and not in
        // payload. This is to be fixed.
        if(status < 0)
        begin
        	l.fail("No response from CPU 0 within 100us");
        	return;
        end else begin

          if ( tmp.data[0] != MSTRFIP_REP_BITRATE )
          begin
            l.fail($sformatf("  Incorrect response from the RT, expected 0x10, got 0x%x", tmp.data[0] ) );
            return;
          end
//      response.data_unpack();
//      l.msg(0, $sformatf("Got bitrate from CPU0: %d", response.data[0]));
        end
        l.pass();
    endtask

    ////////////////////
    // Tasks for SPI  //
    ////////////////////

    task sending_mosi();
        int i;

        mosi_data = $urandom_range(1000,6000);
        $display("MOSI data = %h",mosi_data);
        for(i=32;i>0;i--) begin
            @(posedge spi_clk);
            spi_mosi <= mosi_data[i-1];
        end
    endtask // sending mosi

    task write_to_mt();
        $display("Sending MOSI to the SPI Slave");
        sending_mosi();
    endtask //write_to_mt


    task spi_testing();
        $display("**** SPI Transaction Starts ****");
        spi_cs_n <= 1'b0;
        $display("Chip Select de-asserted");
        write_to_mt();
    endtask // spi_testing

    always @(posedge spi_clk) begin
        test_miso_data <= {test_miso_data[30:0], spi_miso};
    end

    task automatic loadCPUFirmwares(MasterfipDriver drv);
        automatic Logger l = Logger::get();
        const string cpu0_fw = "masterfip-rt-ba.bin"; //"masterfip-rt-ba-sim.bin";
        const string cpu1_fw = "masterfip-rt-cmd.bin";
        l.startTest("Load CPU Firmwares & check CPU communication");
        drv.m_mt.rom.pretty_print();
      
        if (!g_fast_load_cpu_firmware)
        begin
        	  l.msg(0, $sformatf("Loading '%s' to CPU0", cpu0_fw) );
        	  drv.m_mt.load_firmware(0, cpu0_fw);
        	  l.msg(0, $sformatf("Loading '%s' to CPU1", cpu1_fw) );
            drv.m_mt.load_firmware(1, cpu1_fw);
        end
        l.msg(0, "Resetting CPUs");
        drv.m_mt.reset_core(0, 0);
        drv.m_mt.reset_core(1, 0);
        #100us; // give some time for the firmware to boot up
        checkCPUCommunication(drv);
    endtask // loadCPUFirmwares

    task sending_first_from_ertec();
        int i;

        $display("**** Starting the 1st frame like ERTEC ****");

        mosi_data = 'h00020600; // Basic Info
        for (i=0;i<32;i++)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[31-i];
        end
      
        mosi_data = 'h040000AA;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h040000BB;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h040000CC;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000011;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000022;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000033;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000044;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000055;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h04000066;
        for (i=32;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
        end
        @(posedge spi_clk);
    
    endtask // sending_first_from_ertec

    task sending_second_from_ertec();
        int i;
        $display("**** Starting the 2nd frame like ERTEC ****");
        mosi_data = 'h01020400;  // Basic Info
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A0000AA;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A0000BB;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A0000CC;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A000011;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A000022;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A000033;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        mosi_data = 'h0A000044;
        for (i=32;i>0;i--)
        begin
          @(posedge spi_clk);
          spi_mosi = mosi_data[i-1];
        end
        //  @(posedge spi_clk);
        /*  mosi_data = 'h0A000055;
          for (i=32;i>0;i--)
          begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
          end
        //  @(posedge spi_clk);
          mosi_data = 'h0A000066;
          for (i=32;i>0;i--)
          begin
            @(posedge spi_clk);
            spi_mosi = mosi_data[i-1];
          end
          */
          @(posedge spi_clk);
        
    endtask // sending_second_from_ertec()

    task sending_third_from_ertec();
      int i;

      $display("**** Starting the 3rd frame like ERTEC ****");

      mosi_data = 'h00040200;   // Basic Info
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0B0000AA;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0B0000BB;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0B0000CC;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0B000011;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0B000022;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end
      @(posedge spi_clk);

    endtask

    task sending_fourth_from_ertec();
      int i;

      $display("**** Starting the 4th frame like ERTEC ****");

      mosi_data = 'h01040600; // Basic Info
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C0000AA;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C0000BB;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C0000CC;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000011;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000022;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000033;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000044;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000055;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0C000066;
      for (i=32;i>0;i--)
      begin
      @(posedge spi_clk);
      spi_mosi = mosi_data[i-1];
      end
      @(posedge spi_clk);

    endtask

    task sending_fifth_from_ertec();
        int i;

        $display("**** Starting the 5th frame like ERTEC ****");

        mosi_data = 'h00060400;   // Basic Info
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D0000AA;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D0000BB;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D0000CC;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000011;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000022;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000033;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000044;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000055;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end

        mosi_data = 'h0D000066;
        for (i=32;i>0;i--)
        begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
        end
        @(posedge spi_clk);

    endtask //sending_fifth_from_ertec()

    task sending_sixth_from_ertec();
      int i;

      $display("**** Starting the 6th frame like ERTEC ****");

      mosi_data = 'h01060200;   // Basic Info
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E0000AA;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E0000BB;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E0000CC;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000011;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000022;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000033;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000044;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000055;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end

      mosi_data = 'h0E000066;
      for (i=32;i>0;i--)
      begin
        @(posedge spi_clk);
        spi_mosi = mosi_data[i-1];
      end
      @(posedge spi_clk);

    endtask


    CBusAccessor_VME64x acc;

    // counter that counts the negedge of spi_clk
    always @(negedge spi_clk)
    begin
        if (spi_cs_n)
            negedge_cnt <= 0;
        else
            negedge_cnt <= negedge_cnt + 1;
    end

    // counter that counts the posedge of spi_clk
    always @(posedge spi_clk or posedge spi_cs_n)
    begin
        if (spi_cs_n)
            posedge_cnt <= 0;
        else
            posedge_cnt <= posedge_cnt + 1;
    end

    logic [7:0]  operation;
    logic [7:0]  data_len;
    logic [7:0]  rmq_id;
    logic [31:0] basic_info;

    task write_basic_info();
        int i;

        operation = 'h00;
        rmq_id    = $urandom_range(0,6);
        data_len  = $urandom_range(1,16);
        basic_info= {operation, rmq_id, data_len, 8'h00};

        // This generates the basic info
        
        // First comes the operation
        for (i=8;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = operation[i-1];
        end
        // Then the RMQ_ID
        for (i=8;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = rmq_id[i-1];
        end
        // Then the data length of the payload
        for (i=8;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = data_len[i-1];
        end
        // Then the unused byte that concludes the basic info
        for (i=8;i>0;i--)
        begin
            @(posedge spi_clk);
            spi_mosi = 'h0;
        end
        $display("BASIC INFO : write, rmq_id = %d, data_length = %d\n", rmq_id, data_len);
        q_mosi_data.push_back(basic_info);
        @(posedge DUT.cmp_mt_profip_translator.s_wr_data_valid);
        assert (q_mosi_data[0] == DUT.cmp_mt_profip_translator.s_data_reg_o)
        else $error("Mismatch in the Basic Info of the frame, expected : %h, received : %h ",q_mosi_data[0], DUT.cmp_mt_profip_translator.s_data_reg_o);
    endtask // write_basic_info

    task write_to_rmq();
        int i,j;

        write_basic_info();

        // First send the 3 header data
        for (j=3;j>0;j--)
        begin
            mosi_data = $urandom();
            for (i=32;i>0;i--)
            begin
                @(posedge spi_clk);
                spi_mosi = mosi_data[i-1];
            end
            $display("Header %d = %h",j, mosi_data);
            q_mosi_data.push_back(mosi_data);
        end

        // Then send the payload data
        for (j=data_len;j>0;j--)
        begin
            mosi_data = $urandom();
            for (i=32;i>0;i--)
            begin
                @(posedge spi_clk);
                spi_mosi = mosi_data[i-1];
            end
            $display("Data %d = %h",j, mosi_data);
            q_mosi_data.push_back(mosi_data);
        end
        @(posedge spi_clk);

    endtask // write_to_rmq

    // Main testbench
    initial begin
        automatic MockTurtleDriver drv;
        automatic MasterfipDriver fip; 
        automatic Logger l = Logger::get();

        acc = new(VME.tb);
        /* map func0 to 0x80000000, A32 */
        acc.write('h7ff63, 'h80, A32|CR_CSR|D08Byte3);
        acc.write('h7ff67, 0, CR_CSR|A32|D08Byte3);
        acc.write('h7ff6b, 0, CR_CSR|A32|D08Byte3);
        acc.write('h7ff6f, 36, CR_CSR|A32|D08Byte3);
        acc.write('h7ff33, 1, CR_CSR|A32|D08Byte3);
        acc.write('h7fffb, 'h10, CR_CSR|A32|D08Byte3); /* enable module (BIT_SET = 0x10) */

        acc.set_default_modifiers(A32 | D32 | SINGLE);
        acc.set_default_xfer_size(A32 | D32 | SINGLE);

        drv = new (acc, `MT_BASE, IrqMonitor);
        fip = new (drv);


        drv.init();
        drv.enable_hmqi_irq ( 0, 0, 1 );
        drv.enable_console_irq ( 0, 1 );
        drv.enable_console_irq ( 1, 1 );
        $display("Before loading the firmware!");

        loadCPUFirmwares(fip);
        $display("End of loading firmware");

        spi_cs_n = 'b0;
        write_to_rmq();
        spi_cs_n = 'b1;
        q_mosi_data.delete();
        #1ms;

        spi_cs_n = 'b0;
        write_to_rmq();
        spi_cs_n = 'b1;
        q_mosi_data.delete();
        #1ms;


/*
        spi_cs_n = 'b0;
        sending_first_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0; 
        sending_second_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0;
        sending_third_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0;
        sending_fourth_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0;
        sending_fifth_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0;
        sending_sixth_from_ertec();
        spi_cs_n = 'b1;
        #1ms;

        spi_cs_n = 'b0;
        sending_sixth_from_ertec();
        spi_cs_n = 'b1;
        #1ms;
*/    
        $finish;

    end
 

endmodule // main

